{% extends "layout.html" %}
{% set title = "Gallery" %}


{% block body %}

<h3>Click on any image to see source code</h3>
<br/>

<a href="examples/drawing/atlas.html"><img src="_static/examples/atlas_thumb.png" border="0" alt="atlas"/></a>

<a href="examples/drawing/chess_masters.html"><img src="_static/examples/chess_masters_thumb.png" border="0" alt="chess_masters"/></a>

<a href="examples/drawing/circular_tree.html"><img src="_static/examples/circular_tree_thumb.png" border="0" alt="circular_tree"/></a>

<a href="examples/drawing/degree_histogram.html"><img src="_static/examples/degree_histogram_thumb.png" border="0" alt="degree_histogram"/></a>

<a href="examples/drawing/edge_colormap.html"><img src="_static/examples/edge_colormap_thumb.png" border="0" alt="edge_colormap"/></a>

<a href="examples/drawing/ego_graph.html"><img src="_static/examples/ego_graph_thumb.png" border="0" alt="ego_graph"/></a>

<a href="examples/drawing/four_grids.html"><img src="_static/examples/four_grids_thumb.png" border="0" alt="four_grids"/></a>

<a href="examples/drawing/giant_component.html"><img src="_static/examples/giant_component_thumb.png" border="0" alt="giant_component"/></a>

<a href="examples/drawing/house_with_colors.html"><img src="_static/examples/house_with_colors_thumb.png" border="0" alt="house_with_colors"/></a>

<a href="examples/drawing/knuth_miles.html"><img src="_static/examples/knuth_miles_thumb.png" border="0" alt="knuth_miles"/></a>

<a href="examples/drawing/labels_and_colors.html"><img src="_static/examples/labels_and_colors_thumb.png" border="0" alt="labels_and_colors"/></a>

<a href="examples/drawing/lanl_routes.html"><img src="_static/examples/lanl_routes_thumb.png" border="0" alt="lanl_routes"/></a>

<a href="examples/drawing/node_colormap.html"><img src="_static/examples/node_colormap_thumb.png" border="0" alt="node_colormap"/></a>

<a href="examples/drawing/random_geometric_graph.html"><img src="_static/examples/random_geometric_graph_thumb.png" border="0" alt="random_geometric_graph"/></a>

<a href="examples/drawing/sampson.html"><img src="_static/examples/sampson_thumb.png" border="0" alt="sampson"/></a>

<a href="examples/drawing/simple_path.html"><img src="_static/examples/simple_path_thumb.png" border="0" alt="simple_path"/></a>

<a href="examples/drawing/unix_email.html"><img src="_static/examples/unix_email_thumb.png" border="0" alt="unix_email"/></a>

<a href="examples/drawing/weighted_graph.html"><img src="_static/examples/weighted_graph_thumb.png" border="0" alt="weighted_graph"/></a>

{% endblock %}
