<h3>Download</h3>
{% if version.split('.')[-1].startswith('dev') %}
<p>This documentation is for version <b>{{ version }}</b>, which is
  not released yet.</p>
<p>You can get the latest source from 
  <a href="http://networkx.lanl.gov/hg/networkx/">http://networkx.lanl.gov/hg/networkx/</a> or look for
  released versions in the <a href="http://pypi.python.org/pypi/networkx">Python
    Package Index</a>.</p>
{% else %}
<p>Current version: <b>{{ version }}</b></p>
<p>Get NetworkX from the <a href="http://pypi.python.org/pypi/networkx">Python Package
Index</a>, or install it with:</p>
<pre>easy_install networkx</pre>
{% endif %}

<h3>Questions? Suggestions?</h3>

<p>Join the <a href="http://groups.google.com/group/networkx-discuss">Google group</a>:</p>
<form action="http://groups.google.com/group/networkx-discuss/boxsubscribe"
      style="padding-left: 1em">
  <input type="text" name="email" value="your@email"/>
  <input type="submit" name="sub" value="Subscribe" />
</form>
<p>You can also open a
<a href="http://networkx.lanl.gov/trac/newticket">ticket</a>
 at the <a href="http://networkx.lanl.gov/trac">NetworkX Developer Zone</a>.</p>
