

/** Link styles */
:link, :visited {
 text-decoration: none;
 color: #CA7900;
 border-bottom: 1px dotted #bbb;
}

:link:hover, :visited:hover { background-color: white; color: #2491CF; }
h1 :link, h1 :visited ,h2 :link, h2 :visited, h3 :link, h3 :visited,
h4 :link, h4 :visited, h5 :link, h5 :visited, h6 :link, h6 :visited {
 color: inherit;
}
.trac-rawlink { background-color: #*BFD1D4; border-bottom: none } 


h1 {
    margin: 0;
/*    padding: 0.7em 0 0.3em 0; */
    font-size: 1.5em; 
    color: #11557C;
}


/* Header */
#header hr { display: none }
#header h1 { margin: 1.5em 0 -1.5em; }
#header img { border: none; margin: 0 0 -3em }
#header :link, #header :visited, #header :link:hover, #header :visited:hover {
 background: transparent;
 color: #11557C;
 margin-bottom: 2px;
 border: none;
 font-size: 1.5em;

}
#header h1 :link:hover, #header h1 :visited:hover { color: #000 }

dt :link:hover, dt :visited:hover { background-color: #BFD1D4; color: #000 }
dt em {
    border-bottom: 1px dotted #bbb;
    color: #CA7900;
    font-style: normal;
    text-decoration: none;
}


.milestone .info h2 em { color: #CA7900; font-style: normal }

table.progress td { background: #fff; padding: 0 }
table.progress td.new { background: #f5f5b5 }
table.progress td.closed { background: #11557C}
table.progress td :hover { background: none }

/* Main navigation bar */
#mainnav {
 border: 1px solid #000;
 font: normal 10px verdana,'Bitstream Vera Sans',helvetica,arial,sans-serif;
 margin: .66em 0 .33em;
 padding: .2em 0;
}
#mainnav li { border-right: none; padding: .25em 0 }
#mainnav :link, #mainnav :visited {
 border-right: 1px solid #fff;
 border-bottom: none;
 border-left: 1px solid #555;
 color: #CA7900; /* #11557C; */
 padding: .2em 20px;
}
* html #mainnav :link, * html #mainnav :visited { background-position: 1px 0 }
#mainnav :link:hover, #mainnav :visited:hover {
 background-color: #ccc;
 border-right: 1px solid #ddd;
}
#mainnav .active :link, #mainnav .active :visited {
 border-top: none;
 border-right: 1px solid #000;
 border-left: 1px solid #000;
 color:  #CA7900; 
 font-weight: normal; 
}
#mainnav .active :link:hover, #mainnav .active :visited:hover {
 border-right: 1px solid #000;
  background-color: white; 
}


#metanav {  background-color: white; }
#main {  background-color: white; }
/*#content { padding-bottom: 2em; position: relative; background-color: white;  } */
#altlinks { clear: both; text-align: center; background-color: white; }
#footer {  background-color: white; }
#sitefooter {  background-color: white; }
#siteheader {  background-color: white; }
#header {  background-color: white; }
#banner {  background-color: white; }
#main {  background-color: white; }

