"""
*********
JSON data 
*********
Generate and parse JSON serializable data for NetworkX graphs.
"""
from networkx.readwrite.json_graph.node_link import *
from networkx.readwrite.json_graph.adjacency import *
from networkx.readwrite.json_graph.tree import *
from networkx.readwrite.json_graph.serialize import *
